/// @file
////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2016/17      Christian Lessig, Otto-von-Guericke Universitaet Magdeburg
///
////////////////////////////////////////////////////////////////////////////////////////////////////
///
///  module     : lecture 6
///
///  author     : lessig@isg.cs.ovgu.de
///
///  project    : GPU Programming
///
///  description: race conditions
///
////////////////////////////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <vector>
#include <thread>
#include <cstdlib>
#include <mutex>


////////////////////////////////////////////////////////////////////////////////////////////////////
// swap buffers
////////////////////////////////////////////////////////////////////////////////////////////////////
void
swapBuffers( int* buf1, int* buf2, const int n,
             std::mutex& mutex_buf1, std::mutex& mutex_buf2)
{
  mutex_buf1.lock();
  mutex_buf2.lock();

  for( int i = 0; i < n; ++i) {
    std::swap( buf1[i], buf2[i]);
  }

  mutex_buf1.unlock();
  mutex_buf2.unlock();
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// program entry point
////////////////////////////////////////////////////////////////////////////////////////////////////
int
main( int /*argc*/, char** /*argv*/ ) {

  const int n = 8192;
  int buffer1[n];
  int buffer2[n];
  // int buffer3[n];

  std::mutex mutex_buffer1;
  std::mutex mutex_buffer2;

  std::thread t1( swapBuffers, buffer1, buffer2, n,
                  std::ref(mutex_buffer1), std::ref(mutex_buffer2));
  std::thread t2( swapBuffers, buffer2, buffer1, n,
                  std::ref(mutex_buffer2), std::ref(mutex_buffer1));

  t1.join();
  t2.join();

  return EXIT_SUCCESS;
}
