/// @file
////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2016/17      Christian Lessig, Otto-von-Guericke Universitaet Magdeburg
///
////////////////////////////////////////////////////////////////////////////////////////////////////
///
///  module     : lecture 6
///
///  author     : lessig@isg.cs.ovgu.de
///
///  project    : GPU Programming
///
///  description: race conditions
///
////////////////////////////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <vector>
#include <algorithm>

#include <omp.h>

////////////////////////////////////////////////////////////////////////////////////////////////////
// Compute large prime
////////////////////////////////////////////////////////////////////////////////////////////////////
template<typename T>
void
multMatrixVector( const int n, const T mat, const T vec, T res) {

  std::fill( res.begin(), res.end(), 0);

  #pragma omp parallel
  for( int i = 0; i < n; ++i) {
    for( int j = 0; j < n; ++j) {
      res[i] += mat[i*n+j] * vec[j];
    }
  }

}

////////////////////////////////////////////////////////////////////////////////////////////////////
// program entry point
////////////////////////////////////////////////////////////////////////////////////////////////////
int
main( int /*argc*/, char** /*argv*/ ) {

  const int n = 2 * 8192;

  std::vector<int> mat( n*n);
  std::vector<int> vec( n);
  std::vector<int> res( n);

  std::generate( mat.begin(), mat.end(), std::rand);
  std::generate( vec.begin(), vec.end(), std::rand);

  multMatrixVector( n, mat, vec, res);

  return EXIT_SUCCESS;
}
