/// @file
////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2017/18      Christian Lessig, Otto-von-Guericke Universitaet Magdeburg
///
////////////////////////////////////////////////////////////////////////////////////////////////////
///
///  module     : tutorial 4
///
///  author     : lessig@isg.cs.ovgu.de
///
///  project    : GPU Programming
///
///  description: race conditions
///
////////////////////////////////////////////////////////////////////////////////////////////////////

// includes, system
#include <iostream>
#include <iomanip>
#include <vector>
#include <thread>
#include <cstdlib>
#include <cmath>
#include <mutex>

int n = 15;
std::mutex mutex_outer;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Parallel worker for norm computation
////////////////////////////////////////////////////////////////////////////////////////////////////
template< typename T >
void
computeOuterWorker( unsigned int tid, unsigned int /*num_threads*/,
                    const std::vector<T>& /*lhs*/, const std::vector<T>& /*rhs*/,
                    std::vector< std::vector<T> >& outer )
{
  try {
    std::lock_guard<std::mutex> lg(mutex_outer);
    outer[tid].resize( 2 << (2*n));
  }
  catch( std::exception& e) {
    std::cout << "e : " << typeid(e).name() << " :: " << e.what() << std::endl;
  }

  // compute outer product
  // ...
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// program entry point
////////////////////////////////////////////////////////////////////////////////////////////////////
int
main( int /*argc*/, char** /*argv*/ ) {

  typedef double DataType;
  std::vector<DataType> lhs( 2 << n, 1.0);
  std::vector<DataType> rhs( 2 << n, 2.0);
  std::vector< std::vector<DataType> > outer( 2);

  std::thread t1(computeOuterWorker<DataType>, 0, 2, std::ref(lhs), std::ref(rhs), std::ref(outer));
  std::thread t2(computeOuterWorker<DataType>, 1, 2, std::ref(lhs), std::ref(rhs), std::ref(outer));

  // do useful stuff

  t1.join();
  t2.join();

  // process outer products


  return EXIT_SUCCESS;
}
